<?php

/**
 * @file
 * Administrative page callbacks for Paddle Core module.
 */

/**
 * Form builder for setting the active homepage.
 */
function paddle_core_site_settings_form($form, &$form_state) {
  $form = array();

  if (module_exists('paddle_contextual_toolbar')) {
    $form['#after_build'] = array(
      'paddle_core_add_contextual_actions_site_settings_form',
    );
  }

  $form['website_info'] = array(
    '#type' => 'fieldset',
    '#title' => t('Website info'),
  );
  // Provide a field to change the Site name.
  $form['website_info']['site_name'] = array(
    '#type' => 'textfield',
    '#title' => t('Site name'),
    '#required' => TRUE,
    '#default_value' => variable_get('site_name', 'Paddle'),
  );

  // Provide a field to change the Site homepage and error page.
  $pages = array(
    'site_frontpage' => t('Active homepage'),
    'site_403' => t('403 (access denied) page'),
    'site_404' => t('404 (not found) page'),
  );
  foreach ($pages as $var_name => $page_title) {
    $page = 'node';
    // Generate user-friendly value in format "Node title (node/1)" from
    // "node/1".
    if (variable_get($var_name, '')) {
      $matches = array();
      $result = preg_match('/node\/([0-9]+)/',
        variable_get($var_name, ''), $matches);
      if ($result > 0) {
        $node = node_load($matches[$result]);
        if ($node) {
          $page = check_plain($node->title) . " (node/$node->nid)";
        }
      }
    }
    $form['website_info'][$var_name] = array(
      '#type' => 'textfield',
      '#title' => $page_title,
      '#required' => TRUE,
      '#default_value' => $page,
      '#size' => 60,
      '#autocomplete_path' => 'paddle_site_settings_autocomplete',
    );
  }

  // Provide a field to change the Site e-mail.
  $form['website_info']['site_mail'] = array(
    '#type' => 'textfield',
    '#title' => t('E-mail address'),
    '#default_value' => variable_get('site_mail', ini_get('sendmail_from')),
    '#description' => t("The <em>From</em> address in automated e-mails sent during registration and new password requests, and other notifications. (Use an address ending in your site's domain to help prevent this e-mail being flagged as spam.)"),
    '#required' => TRUE,
  );

  $form['social_media_sharing'] = array(
    '#type' => 'fieldset',
    '#title' => t('Social Media Sharing'),
  );

  $form['social_media_sharing']['default_shared_image'] = array(
    '#title' => t('Choose a default image'),
    '#type' => 'paddle_scald_atom',
    '#button_text' => t('Choose an image...'),
    '#default_value' => variable_get('paddle_default_sharing_image', NULL),
    '#allowed_types' => array('image'),
  );

  $form['social_media_sharing']['default_shared_image']['description'] = array(
    '#markup' => t('The image which is used when visitors share a page on social media'),
    '#prefix' => '<div class="content-type-description">',
    '#suffix' => '</div>',
    '#weight' => 10
  );

  $form['maintenance_mode'] = array(
    '#type' => 'fieldset',
    '#title' => t('Maintenance mode'),
  );

  $form['maintenance_mode']['paddle_maintenance_mode'] = array(
    '#type' => 'radios',
    '#title' => t('Enable maintenance mode'),
    '#options' => array(1 => t('Yes'), 0 => t('No')),
    '#default_value' => variable_get('paddle_maintenance_mode', 0),
  );

  $form['maintenance_mode']['paddle_maintenance_mode_message'] = array(
    '#type' => 'textarea',
    '#title' => t('Message to anonymous users'),
    '#default_value' => variable_get('paddle_maintenance_mode_message', t('Behind the scenes we are building a brand new website.<br>Check back soon! Need more info? Paddle to <a href="http://kanooh.be">kanooh.be</a>!')),
    '#description' => t("You can use <em>&lt;br&gt;</em> to put a new line in the message. You can also use <em>&lt;a&gt;</em> to add a link, for example <em>&lt;a href=\"http://kanooh.be\"&gt;kanooh.be&lt;a&gt;</em>."),
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save configuration'),
  );

  return $form;
}

/**
 * Validate callback for the Site Settings form.
 */
function paddle_core_site_settings_form_validate($form, &$form_state) {
  // Validate the e-mail address.
  if ($error = user_validate_mail($form_state['values']['site_mail'])) {
    form_set_error('site_mail', $error);
  }
}

/**
 * Submit callback for the Site Settings form.
 */
function paddle_core_site_settings_form_submit($form, &$form_state) {
  $old_default_sharing_image = variable_get('paddle_default_sharing_image', NULL);
  // Save the site name.
  variable_set('site_name', $form_state['values']['site_name']);

  // Save the autocomplete pages - frontpage and error pages.
  foreach (array('site_frontpage', 'site_403', 'site_404') as $var_name) {
    $page = '';
    $matches = array();
    $result = preg_match('/ \(node\/([0-9]+)\)$/', $form_state['values'][$var_name], $matches);
    if ($result > 0) {
      // It is indeed a value that looks like "node title (node/999)".
      $page = 'node/' . $matches[$result];
    }

    if ($page) {
      variable_set($var_name, $page);
    }
  }

  // Save the site e-mail.
  variable_set('site_mail', $form_state['values']['site_mail']);

  // We need to clear the page cache in order to pick up new access but only if
  // the value has been changed.
  if ($form_state['values']['paddle_maintenance_mode'] !== variable_get('paddle_maintenance_mode', 0) ||
    $form_state['values']['paddle_maintenance_mode_message'] !== variable_get('paddle_maintenance_mode_message', '')
  ) {
    cache_clear_all();
  }
  // Save the maintenance settings.
  variable_set('paddle_maintenance_mode', $form_state['values']['paddle_maintenance_mode']);
  variable_set('paddle_maintenance_mode_message', $form_state['values']['paddle_maintenance_mode_message']);

  // Save the default sharing image.
  variable_set('paddle_default_sharing_image', $form_state['values']['default_shared_image']);
  drupal_set_message(t('The configuration options have been saved.'), 'status');

  // If a new default sharing image has been uploaded or the existing one has
  // been overwritten, all page caches need to be cleared.
  if (!empty($old_default_sharing_image)) {
    if (empty($form_state['values']['default_shared_image']) || $old_default_sharing_image != $form_state['values']['default_shared_image']) {
      cache_clear_all(NULL, 'cache_page');
    }
  }
}

/**
 * After-build of the paddle_core_add_contextual_actions_site_settings_form.
 */
function paddle_core_add_contextual_actions_site_settings_form($form) {
  ctools_include('ajax');
  ctools_include('modal');
  ctools_modal_add_js();
  paddle_contextual_toolbar_add_js('click_delegator');

  $actions = array();

  if (isset($form['submit'])) {
    $actions[] = array(
      'action' => l(t('Save'), '', array(
        'attributes' => array(
          'class' => array('ui-icon', 'fa-floppy-o'),
          'data-paddle-contextual-toolbar-click' => $form['submit']['#id'],
        ),
        'html' => TRUE,
        'external' => TRUE,
      )),
      'class' => array('save'),
      'weight' => 0,
    );
  }

  paddle_contextual_toolbar_actions($actions);

  return $form;
}
